#<profile4xi profile = "03" />
#<program4xi program = "dupsocket4xi.py">
#<abstract4xi abstract = "providing class DupSocket4Xi"/>
#<refer4xi reference = "reduction.py"/>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'DupSocket4Xi']
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                    # system ones
    import os
    import sys
    import signal
except:
    pass
try:                    # threading ones       
    import threading                
    import _thread   
except:
    pass
try:                    # winapi
    import _winapi
    from _winapi import CloseHandle
    from _winapi import DuplicateHandle
    from _winapi import DUPLICATE_SAME_ACCESS
    from _winapi import CreateProcess
    from _winapi import GetCurrentProcess
    from _winapi import GetExitCodeProcess
    from _winapi import INFINITE
    from _winapi import TerminateProcess
    from _winapi import WaitForSingleObject
    from _winapi import WAIT_OBJECT_0
    from _winapi import WAIT_TIMEOUT    
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             
    from multiprocessing.reduction import DupSocket
except:
    print("dupsocket4xi.py: *** failed in from multiprocessing.reduction import DupSocket ***")
        
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

from examine4xi import examine4xi

from resourcesharer4xi import ResourceSharer4Xi 

from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi   

#"
#/>

# =================================================================================================
#<class4xi class = "DupSocket4Xi" base = "object" child = "None" synopsis = "duplicates socket with given socket" usage = "
#   reduce_socket4xi(s):
#        return rebuild_socket4xi, (DupSocket4Xi(s),)
#>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class DupSocket4Xi(object): 
    '''
    DupSocket4Xi class duplicates socket with given socket
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('detach','sense', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    resource_sharer2xi = ResourceSharer4Xi()
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DupSocket4Xi" usage = "initializes DupSocket4Xi with (self, sock)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, sock):
        '''
        initializes DupSocket4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/10)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupSocket4Xi"
        method2print = "__init__"
        message2print = "{(Step 1/10a) started" + "; init-of-DupSocket4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
                                  
        message2print = "{(Step 1/10b) passed" + "; sock-passed-to-init-of-DupSocket4Xi: " + str(sock) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
            
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-DupSocket-init-performing-by-duplicate4xi: " + str(DupSocket.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
            DupSocket.__init__(self, sock)
            
            message2print = "{(StepOriginal) performed" + "; Original-DupSocket-init-performed-by-duplicate4xi: " + str(DupSocket.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
        else:        
                          
            # -----------------------------------------------------------------------------------------
            # (Step 2/10)            
            # -----------------------------------------------------------------------------------------    
            message2print = "{(Step 2/10a) performing" + "; winapi-OpenProcess-performing-by-init-of-DupSocket4Xi: " + str(_winapi.OpenProcess) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
             
            new_sock = None
                            
            try:
                message2print = "{(Step 2/10b) performing" + "; dup-performing-by-init-of-DupSocket4Xi: " + str(sock.dup) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                new_sock = sock.dup()
                
                message2print = "{(Step 2/10c) performed" + "; dup-performed-by-init-of-DupSocket4Xi: " + str(sock.dup) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                message2print = "{(Step 2/10d) got" + "; new_sock-got-by-init-of-DupSocket4Xi: " + str(new_sock) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                
            except:
                message2print = "{(Step 2/10z1) failed" + "; new_sock-failed-by-init-of-DupSocket4Xi: " + str(new_sock) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
            # -----------------------------------------------------------------------------------------
            # (Step 3/10)            
            # -----------------------------------------------------------------------------------------    
            if new_sock is None:
                message2print = "{(Step 3/10a) failed" + "; new_sock-failed-by-init-of-DupSocket4Xi: " + str(new_sock) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                                   
            else:
                # ---------------------------------------------------------------------------------------------
                #<function4xi function = "send-handle4xi" usage = "
                #    self._id = resource_sharer2xi.register(send, new_sock.close)
                #"> 
                #<note4xi note = "internal function in __init__ method of DupSocket4Xi class"/>
                #<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
                # ---------------------------------------------------------------------------------------------
                def send(connection, pid):
                    message2print = "{(Step 0/1) passed" + "; connection-passed-to-send-of-init-of-DupSocket4Xi: " + str(connection) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    message2print = "{(Step 0/2) passed" + "; pid-passed-to-send-of-init-of-DupSocket4Xi: " + str(pid) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    share = new_sock.share(pid)
                    
                    message2print = "{(Step 0/3) got" + "; share-got-by-send-of-init-of-DupSocket4Xi: " + str(share) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    message2print = "{(Step 0/4) passing" + "; share-passing-to-send-of-connection-of-DupSocket4Xi: " + str(share) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    try:
                    
                        connection.send_bytes(share)
                    
                        message2print = "{(Step 0/5a) performed" + "; send-bytes-performed-by-send-of-connection-of-DupSocket4Xi: " + str(connection.send_bytes) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #
                    except:
                        message2print = "{(Step 0/5z) failed" + "; send-bytes-failed-by-send-of-connection-of-DupSocket4Xi: " + str(connection.send_bytes) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                    
                #</function4xi>
                    
                self._id = None
                message2print = "{(Step 3/10b) set" + "; id-set-by-init-of-DupSocket4Xi to: " + str(self._id) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #   
                    
                try:
                    message2print = "{(Step 3/10c) passing" + "; send-passing-to-resource_sharer2xi-by-init-of-DupSocket4Xi: " + str(send) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    message2print = "{(Step 3/10d) passing" + "; new-sock-close-passing-to-resource_sharer-by-init-of-DupSocket4Xi: " + str(new_sock.close) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 
                    
                    self._id = resource_sharer2xi.register(send, new_sock.close)
                    
                    message2print = "{(Step 3/10e) got" + "; id-got-by-init-of-DupSocket4Xi: " + str(self._id) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:
                    message2print = "{(Step 3/10z1) failed" + "; id-failed-by-init-of-DupSocket4Xi: " + str(self._id) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                        
    
            # -----------------------------------------------------------------------------------------    
            # (Step 10/10) 
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 10/10a) activated" + "; dubsocket2xi-of-DupSocket4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #                 
            
            message2print = "{(Step 10/10b) inherited" + "; object-inherited-by-dubsocket2xi-of-DupSocket4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #    
             
        
            message2print = "{(Step 10/10c) returning" + "; init-of-DupSocket4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #                        
                    
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "detach-of-DupSocket4Xi" usage = "retrieves handle from process which currently owns it">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def detach(self):
        '''
        retrieves handle from process which currently owns it
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/10)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupSocket4Xi"
        method2print = "detach"
        
        message2print = "{(Step 1/10a) started" + "; detach-of-DupSocket4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) performing" + "; Original-DupSocket-detach-performing-by-duplicate4xi: " + str(DupSocket.detach) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
            return DupSocket.detach(self)

        else:          
                                                        
            # -----------------------------------------------------------------------------------------
            # (Step 2/10)            
            # -----------------------------------------------------------------------------------------                                     
            message2print = "{(Step 2/10a) passing" + "; id-passing-to-resource_sharer-by-detach-of-DupSocket4Xi: " + str(self._id) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            connection2xi = None
            try:
                connection2xi = resource_sharer2xi.get_connection(self._id)
                
                message2print = "{(Step 2/10b) got" + "; connection2xi-got-by-detach-of-DupSocket4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:
                message2print = "{(Step 2/10z1) failed" + "; connection2xi-failed-by-detach-of-DupSocket4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #  
            if connection2xi is None:
                pass
            else:
                # -----------------------------------------------------------------------------------------
                # (Step 3/10)            
                # ----------------------------------------------------------------------------------------- 
                share = None
                return2detach = None            
                try:
                    share = connection2xi.recv_bytes()
                    message2print = "{(Step 3/10a) got" + "; share-got-by-detach-of-DupSocket4Xi: " + str(share) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    message2print = "{(Step 3/10b) passing" + "; share-passing-to-socket-fromshare-by-detach-of-DupSocket4Xi: " + str(share) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #
                    
                    return2detach = socket.fromshare(share)
                    
                    message2print = "{(Step 3/10c) got" + "; return2detach-got-by-detach-of-DupSocket4Xi: " + str(return2detach) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:
                    message2print = "{(Step 3/10z) failed" + "; return2detach-failed-by-detach-of-DupSocket4Xi: " + str(return2detach) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                finally:
                    connection2xi.close()
                      
                message2print = "{(Step 3/10d) returning" + "; return2detach-returning-by-detach-of-DupSocket4Xi: " + str(return2detach) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                    
                return return2detach
            
    #</methodize4xi>    
                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-DupSocket4Xi" usage = "senses DupSocket4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses DupSocket4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        
        class2print = 'DupSocket4Xi'
        method2print = 'sense'
        
           
        message2print = "{(Step 1/5) started" + "; sense-of-DupSocket4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------  
        message2print = "{(Step 2/5) sensing" + "; DupSocket4Xi-sensing-by-sense-of-DupSocket4Xi: " + str(DupSocket4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-DupSocket4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        
      
        message2print = "{(Step 4/5) sensed" + "; DupSocket4Xi-sensed-by-sense-of-DupSocket4Xi: " + str(DupSocket4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-DupSocket4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
    #</methodize4xi>
      

        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-DupSocket4Xi" usage = "reasons DupSocket4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons DupSocket4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
             
        
        class2print = 'DupSocket4Xi'
        method2print = 'reason'

           
        message2print = "{(Step 1/5) started" + "; reason-of-DupSocket4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) reasoning" + "; DupSocket4Xi-reasoning-by-reason-of-DupSocket4Xi: " + str(DupSocket4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 3/5) get reasoned knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-by-reason-of-DupSocket4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 4/5) add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{(Step 4/5) reasoning" + "; DupSocket4Xi-reasoning-by-reason-of-DupSocket4Xi: " + str(DupSocket4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-DupSocket4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb4xi
        
    #</methodize4xi>
         

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-DupSocket4Xi" usage = "aces DupSocket4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces DupSocket4Xi class objects 
        '''
          
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
            
        
        class2print = 'DupSocket4Xi'
        method2print = 'ace'
        
           
        message2print = "{(Step 1/5) started" + "; ace-of-DupSocket4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) acing" + "; DupSocket4Xi-acing-by-ace-of-DupSocket4Xi: " + str(DupSocket4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) get aced knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3/5)got" + "; knowledge-base8xi-got-by-ace-of-DupSocket4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            


        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{(Step 4/5) acing" + "; DupSocket4Xi-acing-by-ace-of-DupSocket4Xi: " + str(DupSocket4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-DupSocket4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
    #</methodize4xi>
#</class4xi>

# =================================================================================================
#<class4xi class = "DupFd4Xi" base = "object" child = "None" synopsis = "Duplicates file descriptor" usage = "
#   reduction.py
#       reduce_socket(s):
#           df = DupFd(s.fileno())
#           return rebuild_socket, (df, s.family, s.type, s.proto)   
#   reduce_socket4xi.py
#       df2xi = DupFd4Xi(s.fileno())
#       return rebuild_socket4xi, (df2xi, s.family, s.type, s.proto) 
#
#>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class DupFd4Xi(object): 
    '''
    Duplicates file descriptor in non-Windows environment
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('detach', 'sense', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    resource_sharer2xi = ResourceSharer4Xi()
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DupFd4Xi" usage = "initializes DupFd4Xi with (self, fd)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, fd):
        '''
        initializes DupFd4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupFd4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/4a) started" + "; init-of-DupFd4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)            
        # -----------------------------------------------------------------------------------------    
        new_fd2xi = os.dup(fd)
        message2print = "{(Step 4/4b) set" + "; new_fd2xi-set-by-dupfd2xi-of-DupFd4Xi to: " + str(new_fd2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #  
        
        # -------------------------------------------------------------------------------------------------
        #<function4xi function = "send" usage = "
        #    self._id = resource_sharer2xi.register(send, close)
        #"> 
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # -------------------------------------------------------------------------------------------------
        def send(conn, pid):
            send_handle(conn, new_fd2xi, pid) 
        #</function4xi>
        
        # -------------------------------------------------------------------------------------------------
        #<function4xi function = "close" usage = "
        #    self._id = resource_sharer2xi.register(send, close)
        #"> 
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # -------------------------------------------------------------------------------------------------        
        def close():
            os.close(new_fd2xi)
        #</function4xi>
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)            
        # -----------------------------------------------------------------------------------------    
        self._id = resource_sharer2xi.register(send, close)
        message2print = "{(Step 3/4) set" + "; id-of-dupfd2xi-set-by-dupfd2xi-of-DupFd4Xi to: " + str(self._id)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #  
                    
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4) 
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 4/4a) activated" + "; dupfd2xi-of-DupFd4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #                 
        
        message2print = "{(Step 4/4b) inherited" + "; object-inherited-by-dupfd2xi-of-DupFd4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #    
                 
        message2print = "{(Step 4/4c) returning" + "; init-of-DupFd4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                        
                
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DupFd4Xi" usage = "detaches DupFd4Xi with (self)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def detach(self):
        '''
        detaches DupFd4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "DupFd4Xi"
        method2print = "detach"
        
        message2print = "{(Step 1/4) started" + "; detach-of-DupFd4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)            
        # -----------------------------------------------------------------------------------------    
        conn2xi = resource_sharer2xi.get_connection(self._id)
        message2print = "{(Step 2/4a) set" + "; conn2xi-set-by-detach-of-DupFd4Xi to: " + str(conn2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #  
        
        try:
            message2print = "{(Step 2/4b) passing" + "; conn2xi-passing-to-recv_handle-by-detach-of-DupFd4Xi: " + str(conn2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #
              
            message2print = "{(Step 2/4c) returning" + "; recv_handle-returning-by-detach-of-DupFd4Xi: " + str(recv_handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #
            
            return recv_handle(conn2xi)
        finally:
            message2print = "{(Step 2/4d) closing" + "; connection-closing-by-detach-of-DupFd4Xi: " + str(conn2xi.close)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #   
        
            conn2xi.close()
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)            
        # -----------------------------------------------------------------------------------------    
        pass        
                    
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4) 
        # -----------------------------------------------------------------------------------------                 
                 
        message2print = "{(Step 4/4) returning" + "; detach-of-DupFd4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                        
                
    #</methodize4xi>    
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of dupsocket4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

